"""quixote.mod_python_handler
$HeadURL: svn+ssh://svn/repos/trunk/quixote/mod_python_handler.py $
$Id: mod_python_handler.py 25422 2004-10-25 21:04:08Z nascheme $

mod_python handler for Quixote.  See the "mod_python configuration"
section of doc/web-server.txt for details.
"""

import sys
from mod_python import apache
from quixote import enable_ptl
from quixote.publish import Publisher
from quixote.config import Config

class ErrorLog:
    def __init__(self, publisher):
        self.publisher = publisher

    def write(self, msg):
        self.publisher.log(msg)

    def close(self):
        pass

class ModPythonPublisher(Publisher):
    def __init__(self, package, **kwargs):
        Publisher.__init__(self, package, **kwargs)
        # may be overwritten
        self.logger.error_log = self.__error_log = ErrorLog(self)
        self.__apache_request = None

    def log(self, msg):
        if self.logger.error_log is self.__error_log:
            try:
                self.__apache_request.log_error(msg)
            except AttributeError:
                apache.log_error(msg)
        else:
            Publisher.log(self, msg)

    def publish_modpython(self, req):
        """publish_modpython() -> None

        Entry point from mod_python.
        """
        self.__apache_request = req
        try:
            self.publish(apache.CGIStdin(req),
                         apache.CGIStdout(req),
                         sys.stderr,
                         apache.build_cgi_env(req))

            return apache.OK
        finally:
            self.__apache_request = None

enable_ptl()

name2publisher = {}

def handler(req):
    opts = req.get_options()
    try:
        package = opts['quixote-root-namespace']
    except KeyError:
        package = None

    try:
        configfile = opts['quixote-config-file']
        config = Config()
        config.read_file(configfile)
    except KeyError:
        config = None

    if not package:
        return apache.HTTP_INTERNAL_SERVER_ERROR

    pub = name2publisher.get(package)
    if pub is None:
        pub = ModPythonPublisher(package, config=config)
        name2publisher[package] = pub
    return pub.publish_modpython(req)
